"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var JSRuleFinder_1 = require("./shared/JSRuleFinder");
var persistPlugins;
var ApolloPlugin = (function () {
    function ApolloPlugin() {
    }
    ApolloPlugin.prototype.configure = function (builder, spin) {
        if (!builder.stack.hasAny('dll') && builder.stack.hasAll(['apollo', 'webpack'])) {
            var persistGraphQL = builder.persistGraphQL && !spin.test;
            if (builder.stack.hasAny(['server', 'web'])) {
                if (!persistPlugins) {
                    var PersistGraphQLPlugin = builder.require('persistgraphql-webpack-plugin');
                    var moduleName = path.resolve('node_modules/persisted_queries.json');
                    if (persistGraphQL) {
                        var clientPersistPlugin = new PersistGraphQLPlugin({
                            moduleName: moduleName,
                            filename: 'extracted_queries.json',
                            addTypename: true
                        });
                        var serverPersistPlugin = new PersistGraphQLPlugin({
                            moduleName: moduleName,
                            provider: clientPersistPlugin
                        });
                        persistPlugins = { client: clientPersistPlugin, server: serverPersistPlugin };
                    }
                    else {
                        var clientPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName });
                        var serverPersistPlugin = new PersistGraphQLPlugin({ moduleName: moduleName });
                        persistPlugins = { client: clientPersistPlugin, server: serverPersistPlugin };
                    }
                }
            }
            builder.config = spin.merge(builder.config, {
                module: {
                    rules: [
                        {
                            test: /\.graphqls/,
                            use: 'raw-loader'
                        },
                        {
                            test: /\.(graphql|gql)$/,
                            exclude: /node_modules/,
                            use: ['graphql-tag/loader'].concat(persistGraphQL ? ['persistgraphql-webpack-plugin/graphql-loader'] : [])
                        }
                    ]
                }
            });
            if (builder.stack.hasAny(['server', 'web'])) {
                var webpack = builder.require('webpack');
                if (persistGraphQL) {
                    var jsRuleFinder = new JSRuleFinder_1.default(builder);
                    var jsRule = jsRuleFinder.findAndCreateJSRule();
                    jsRule.use = spin.merge(jsRule.use, ['persistgraphql-webpack-plugin/js-loader']);
                }
                builder.config = spin.merge(builder.config, {
                    plugins: [
                        new webpack.DefinePlugin({ __PERSIST_GQL__: persistGraphQL }),
                        builder.stack.hasAny('server') ? persistPlugins.server : persistPlugins.client
                    ]
                });
            }
        }
    };
    return ApolloPlugin;
}());
exports.default = ApolloPlugin;
//# sourceMappingURL=ApolloPlugin.js.map